@@ -5,6 +5,7 @@ from __future__ import print_function
 from functools import partial
 
 import tensorflow as tf
+import tensorflow.keras as tfk
 
 from dynastes import activations
 from dynastes.blocks import layer_factory
@@ -28,6 +29,8 @@ class _AttentionBlock1D(DynastesBaseLayer):
                  q_type='Conv1D',
                  k_type=None,
                  v_type=None,
+                 out_type=None,
+                 skip_out=False,
                  num_heads=1,
                  multiquery_attention=False,
                  depth_multiplier=1,
@@ -49,8 +52,11 @@ class _AttentionBlock1D(DynastesBaseLayer):
                  filter_width=2,
                  mask_right=False,
                  add_relative_to_values=False,
+                 scaled=False,
                  return_attn_weights=False,
                  cache_kv=False,
+                 separable_prepointwise=False,
+                 separable_prepointwise_depth='min',
                  **kwargs):
         kwargs['supports_caching'] = True
         super(_AttentionBlock1D, self).__init__(**kwargs)
@@ -68,6 +74,11 @@ class _AttentionBlock1D(DynastesBaseLayer):
         else:
             self.v_type = v_type
 
+        if out_type is None:
+            self.out_type = q_type
+        else:
+            self.out_type = out_type
+
         self.attention_dim = attention_dim
         self.output_dim = output_dim
         self.kernel_size = kernel_size
@@ -93,6 +104,11 @@ class _AttentionBlock1D(DynastesBaseLayer):
         self.return_attn_weights = return_attn_weights
         self.supports_masking = True
         self.cache_kv = cache_kv
+        self.scaled = scaled
+        self.separable_prepointwise = separable_prepointwise
+        self.separable_prepointwise_depth = separable_prepointwise_depth
+        self.skip_out = skip_out
+
         conv_partial = partial(layer_factory.get_1d_layer, kernel_size=kernel_size,
                                grouped=grouped,
                                group_size=group_size,
@@ -100,16 +116,20 @@ class _AttentionBlock1D(DynastesBaseLayer):
                                padding=padding,
                                activation=activation,
                                use_bias=use_bias,
-                               kernel_initializer=self.get_initializer('kernel'),
                                bias_initializer=self.get_initializer('bias'),
                                kernel_regularizer=self.get_regularizer('kernel'),
                                bias_regularizer=self.get_regularizer('bias'),
                                activity_regularizer=None,
                                kernel_constraint=self.get_constraint('kernel'),
-                               bias_constraint=self.get_constraint('bias'))
+                               bias_constraint=self.get_constraint('bias'),
+                               separable_prepointwise=separable_prepointwise,
+                               separable_prepointwise_depth=separable_prepointwise_depth)
         q_filters = attention_dim
         k_filters = attention_dim
-        v_filters = output_dim
+        if skip_out:
+            v_filters = output_dim
+        else:
+            v_filters = attention_dim
 
         if multiquery_attention:
             k_filters //= num_heads
@@ -127,18 +147,50 @@ class _AttentionBlock1D(DynastesBaseLayer):
             attn_strides = strides
             attn_dilation_rate = dilation_rate
 
+        init_stddev = output_dim ** -0.5
+
         self.q_layer = conv_partial(type=self.q_type,
+                                    kernel_initializer=tfk.initializers.RandomNormal(
+                                        stddev=init_stddev * (k_filters ** -0.5)),
                                     filters=q_filters,
                                     strides=q_strides,
-                                    dilation_rate=dilation_rate, name='Conv-Q')
+                                    dilation_rate=dilation_rate, name='Conv-Q',
+                                    wnorm=self.wnorm,
+                                    wlrmul=self.lrmul,
+                                    wgain=self.gain,
+                                    use_wscale=self.use_wscale)
         self.k_layer = conv_partial(type=self.k_type,
+                                    kernel_initializer=tfk.initializers.RandomNormal(stddev=init_stddev),
                                     filters=k_filters,
                                     strides=kv_strides,
-                                    dilation_rate=kv_dilation_rate, name='Conv-K')
+                                    dilation_rate=kv_dilation_rate, name='Conv-K',
+                                    wnorm=self.wnorm,
+                                    wlrmul=self.lrmul,
+                                    wgain=self.gain,
+                                    use_wscale=self.use_wscale)
         self.v_layer = conv_partial(type=self.v_type,
+                                    kernel_initializer=tfk.initializers.RandomNormal(stddev=init_stddev),
                                     filters=v_filters,
                                     strides=kv_strides,
-                                    dilation_rate=kv_dilation_rate, name='Conv-V')
+                                    dilation_rate=kv_dilation_rate, name='Conv-V',
+                                    wnorm=self.wnorm,
+                                    wlrmul=self.lrmul,
+                                    wgain=self.gain,
+                                    use_wscale=self.use_wscale)
+
+        if self.skip_out:
+            self.out_layer = None
+        else:
+            self.out_layer = conv_partial(type=self.out_type,
+                                          kernel_initializer=tfk.initializers.RandomNormal(
+                                              stddev=init_stddev * (output_dim ** -0.5)),
+                                          filters=output_dim,
+                                          strides=1,
+                                          dilation_rate=1, name='Conv-Out',
+                                          wnorm=self.wnorm,
+                                          wlrmul=self.lrmul,
+                                          wgain=self.gain,
+                                          use_wscale=self.use_wscale)
 
         attention_padding = padding
         self.attention_layer = layer_factory.get_1D_attention_layer(
@@ -160,6 +212,7 @@ class _AttentionBlock1D(DynastesBaseLayer):
             mask_right=mask_right,
             filter_width=filter_width,
             add_relative_to_values=add_relative_to_values,
+            scaled=scaled,
         )
 
     def compute_mask(self, inputs, mask=None):
@@ -173,6 +226,8 @@ class _AttentionBlock1D(DynastesBaseLayer):
             'q_type': self.q_type,
             'k_type': self.k_type,
             'v_type': self.v_type,
+            'out_type': self.out_type,
+            'skip_out': self.skip_out,
             'activation': activations.serialize(self.activation),
             'use_bias': self.use_bias,
             'multiquery_attention': self.multiquery_attention,
@@ -198,6 +253,9 @@ class _AttentionBlock1D(DynastesBaseLayer):
             'mask_right': self.mask_right,
             'add_relative_to_values': self.add_relative_to_values,
             'cache_kv': self.cache_kv,
+            'scaled': self.scaled,
+            'separable_prepointwise': self.separable_prepointwise,
+            'separable_prepointwise_depth': self.separable_prepointwise_depth,
         }
         base_config = super(_AttentionBlock1D, self).get_config()
         return {**base_config, **config}
@@ -207,9 +265,12 @@ class _AttentionBlock1D(DynastesBaseLayer):
         ks = self.k_layer.compute_output_shape(input_shape)
         vs = self.v_layer.compute_output_shape(input_shape)
         output_shape = self.attention_layer.compute_output_shape([qs, ks, vs])
+        output_shape, attn_weights_shape = output_shape
+        if not self.skip_out:
+            output_shape = self.out_layer.compute_output_shape(output_shape)
         if self.return_attn_weights:
-            return output_shape
-        return output_shape[0]
+            return [output_shape, attn_weights_shape]
+        return output_shape
 
 
 @tf.keras.utils.register_keras_serializable(package='Dynastes')
@@ -273,7 +334,8 @@ class AttentionBlock1D(_AttentionBlock1D):
         if mask is not None:
             mask = [q_mask, tf.logical_and(k_mask, v_mask)]
         x, weights = self.attention_layer([q, k, v], mask=mask, training=training)
-
+        if not self.skip_out:
+            x = self.out_layer(x, mask=mask, training=training)
         if self.return_attn_weights:
             return x, weights
         return x
@@ -373,6 +435,8 @@ class SelfAttentionBlock1D(AttentionBlock1D):
         if mask is not None:
             mask = [q_mask, tf.logical_and(k_mask, v_mask)]
         x, weights = self.attention_layer([q, k, v], mask=mask, training=training)
+        if not self.skip_out:
+            x = self.out_layer(x, mask=mask, training=training)
         x_shape = t2t_common.shape_list(x)
         if pad_q_to_kv:
             if q_shape[1] != kv_shape[1]:
