@@ -158,7 +158,7 @@ def lsh_attention_back_fn(q, k, v, t_vectors_q, t_group, bucket_length, threshol
                                           dtype=dtype,
                                           t_vectors_y=t_vectors_k)
     attention_weights = tf.nn.softmax(tf.cast(logits, tf.float32), axis=-1)
-    return tf.matmul(attention_weights, v)
+    return tf.cast(tf.matmul(attention_weights, tf.cast(v, tf.float32)), v.dtype)
 
 
 ##
@@ -178,3 +178,25 @@ def lsh_attention(q, k, v, t_vectors_q, t_group, bucket_length, threshold=0.5, d
         return sparse_dense_matmul(attention_weights, v)
     else:
         return tf.matmul(tf.sparse.to_dense(attention_weights), v)
+
+
+def zero_abs_gt(t: tf.Tensor, value):
+    mask = tf.math.is_finite(t)
+    return tf.where(mask, tf.clip_by_value(t, -value, value), tf.zeros_like(t))
+
+
+def zero_abs_gt_grad(t: tf.Tensor, value):
+    with tf.name_scope('zero_abs_gt_grad'):
+        @tf.custom_gradient
+        def func(x):
+            y = zero_abs_gt(x, value)
+
+            @tf.custom_gradient
+            def grad(dy):
+                with tf.name_scope('zero_abs_gt_grad_grad'):
+                    dx = zero_abs_gt(dy, value)
+                    return dx, lambda ddx: zero_abs_gt(ddx, value)
+
+            return y, grad
+
+        return func(t)
