@@ -1,4 +1,6 @@
 from . import attention_nd
+from . import embedding_ops
+from . import image_ops
 from . import localized_attention_nd
 from . import pad_ops
 from . import scale_ops
